%define RSH_COMMAND rsh

Summary: MPI-CH
Name: mpich
Version: 1.1.1
Release: 6
Copyright: Freeware
Group: Beowulf
Packager: Red Hat Contrib|Net <rhcn-bugs@redhat.com>
URL: http://www.mcs.anl.gov/mpi/mpich/
Source: ftp://ftp.mcs.anl.gov/pub/mpi/mpich.tar.gz
Source1: README-alignment
Source2: mpich.csh
Source3: mpich.sh
Patch: ftp://info.mcs.anl.gov/pub/mpi/patch/1.1.1/3522
Patch1: sed-patch
Patch2: bigendian-patch
Patch3: farg-patch
Patch4: mpe-makefile-patch
Patch5: clog-merge-patch
BuildRoot: /var/tmp/mpich-root
Requires: %{RSH_COMMAND}
# Note building source requires: perl >= 5, egcs, egcs-c++, egcs-g77

%description
MPI (which stands for Message Passing Interface) is a standard for
writing message-passing programs.  As such the interface attempts to
establish a practical, portable, efficient, and flexible standard for
message passing.

This version uses %{RSH_COMMAND} as the remote login method.

Linux does not currently support the shared memory driver.

%changelog
* Sun Nov 15 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- RHCN release
- release 6

* Thu Nov 12 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- mpe logging now works
- added profile.d for setting logging to CLOG
- more tidying up

* Wed Nov 04 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- some changes to mpiCC so that mpi2c++ works on sparc
- added patches to try to make logging work
- added the rest of the examples
- minor tidying up
- release 5

* Wed Oct 14 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- added a default machines file (8 times hostname)
- added romio man pages

* Mon Sep 14 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- added README, COPYRIGHT and note on alignment
- added romio documentation

* Thu Sep 10 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- some tidying up
- added RPM_OPT_FLAGS

* Fri Sep 04 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- release 2
- patch to getarg to sort out problems with Fortran codes

* Tue Sep 01 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- finally built on sparc!
- decided that C++ was better packaged separately

* Sat Aug 29 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- fixed sed error in configure file
- single package again

* Fri Aug 28 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- define RSH_COMMAND for easy changing
- corrected copyright
- minor corrections
- added missing header files
- added %clean

* Thu Aug 27 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- now all working, install still a bit hacky
- install jumpshot entirely independently
- please give feedback on this package to me

* Mon Aug 24 1998 Justin Cormack <jpc1@doc.ic.ac.uk>
- added jumpshot support
- added relevant patches for current distribution
- upgraded to MPICH 1.1.1
- split package
- added BuildRoot
- put in /usr/ not /usr/mpich/
- more complete files listing
- added Requires
- tidied up spec file, adding URL etc
- added changelog
- previous packager Erik Arjan Hendriks <hendriks@beowulf.org>

%prep
%setup -n mpich
%patch0 -p0
pushd src/env
patch < %{PATCH3}
popd

# sparc alignment message
%ifarch sparc
cp %{SOURCE1} $RPM_BUILD_DIR/mpich
%endif

%build

# fix the c++ option in configure (gnu sed difference)
patch < %{PATCH1}

# mpe fix
pushd mpe
patch < %{PATCH4}
patch < %{PATCH5}
popd

# You may want to change the options here
# RedHat gcc+egcs requires -cc=egcs to compile
# sparc seems to need -c++ even to include mpiCC

sh ./configure -opt=-O -nodevdebug -rsh=%{RSH_COMMAND} -cc=egcs -c++=g++ \
               -cflags="$RPM_OPT_FLAGS" -fflags="$RPM_OPT_FLAGS"

# patches for logging
%ifarch i386 alpha
pushd lib/LINUX/ch_p4
patch < %{PATCH2}
popd
%endif

make

%install
rm -rf $RPM_BUILD_ROOT
mkdir $RPM_BUILD_ROOT
util/mpiinstall \
          -nonupshot \
          -prefix=$RPM_BUILD_ROOT/usr \
          -manpath=$RPM_BUILD_ROOT/usr/man

# MPI C++ bindings installed in separate mpi2c++ package
rm -rf $RPM_BUILD_ROOT/usr/lib/LINUX/ch_p4/c++

# some of the .h files are missing!
cp -a $RPM_BUILD_DIR/mpich/include/{mpidefs.h,mpio.h} \
      $RPM_BUILD_ROOT/usr/include

# install web documentation
cp -a $RPM_BUILD_DIR/mpich/www \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}

# install user guide and install guide
cp -a $RPM_BUILD_DIR/mpich/doc/{guide,install}.ps.gz \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}

# install README, COPYRIGHT
cp -a $RPM_BUILD_DIR/mpich/{README*,COPYRIGHT} \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}

# install romio documentation
mkdir $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/romio
cp -a $RPM_BUILD_DIR/mpich/romio/doc/*.ps* \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/romio
cp -a $RPM_BUILD_DIR/mpich/romio/{README*,COPYRIGHT} \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/romio
# and romio man pages
cp -a $RPM_BUILD_DIR/mpich/romio/man/man3/* \
      $RPM_BUILD_ROOT/usr/man/man3

# compress man pages
gzip -9 $RPM_BUILD_ROOT/usr/man/man{1,3,4}/*

# fix files which do not like BuildRoot
# This is a bit unpleasant really, as the install program has code
# to get this right first time. Oh well. Also, there are still references
# to the original build directorry in some files, which seems rather nasty,
# so I am fixing these too. 
# also the warnings from -fhandle-exceptions in egcs which is now -fexceptions
# and not needed. Not the best place to change this really.
# Also fix mpiCC so it works with the mpi2c++ package

Fixup() {
    while [ $# -ne 0 ]
    do
        if [ -d $1 ]
        then
            echo 'omitting directory $1'
        else
            mv $1 $1.orig
            sed \
                 -e "s@$RPM_BUILD_ROOT@@g" \
                 -e "s@$RPM_BUILD_DIR@/usr@g" \
                 -e 's/ -fhandle-exceptions//g' \
                 -e 's/cpplib=-lmpich++/cpplib=-lmpi++/g' \
                 -e 's@\(cpplibfullname=\).*@\1/usr/lib/libmpi++.a@g' \
                 -e 's@CXXFLAGS=.*@CXXFLAGS=\"-I/usr/include/mpi2c++\"@g' \
                 $1.orig > $1
            rm -f $1.orig
        fi
        shift
    done
}

pushd $RPM_BUILD_ROOT/usr/lib/LINUX/ch_p4
Fixup chkserv chp4_servs config.status mpiCC mpicc mpif77 mpif90 \
      mpiman mpireconfig mpirun mpirun.args mpirun.ch_p4 mpirun.pg \
      serv_p4 tstmachines
popd

# All the files in /usr/bin are hard links. But we have changed the files
# above. So we had better make them all symlinks. Except tarch, tdevice.
# Maybe these should be symlinked the other way around, in case there are
# more devices. Address this problem when the shared memory device works
# on Linux.

pushd $RPM_BUILD_ROOT/usr/bin/
rm execer mpiCC mpicc mpif77 mpif90 mpiman mpireconfig \
   mpirun mpirun.args mpirun.ch_p4 mpirun.pg serv_p4 tstmachines
ln -s ../lib/LINUX/ch_p4/execer
ln -s ../lib/LINUX/ch_p4/mpiCC
ln -s ../lib/LINUX/ch_p4/mpicc
ln -s ../lib/LINUX/ch_p4/mpif77
ln -s ../lib/LINUX/ch_p4/mpif90
ln -s ../lib/LINUX/ch_p4/mpiman
ln -s ../lib/LINUX/ch_p4/mpireconfig
ln -s ../lib/LINUX/ch_p4/mpirun
ln -s ../lib/LINUX/ch_p4/mpirun.args
ln -s ../lib/LINUX/ch_p4/mpirun.ch_p4
ln -s ../lib/LINUX/ch_p4/mpirun.pg
ln -s ../lib/LINUX/ch_p4/serv_p4
ln -s ../lib/LINUX/ch_p4/tstmachines
popd

# mpif77 puts in a symlink to lib/LINUX/ch_p4/mpif.h, but this is
# installed in the include directory. Better add another symlink.

pushd $RPM_BUILD_ROOT/usr/lib/LINUX/ch_p4
ln -s ../../../include/mpif.h
popd

# And we really just want libraries in /usr/lib
# Alternatively add /usr/lib/LINUX/ch_p4 to /etc/ld.so.conf
# whichever you prefer really.

pushd $RPM_BUILD_ROOT/usr/lib
ln -s LINUX/ch_p4/libampi.a
ln -s LINUX/ch_p4/libfmpich.a
ln -s LINUX/ch_p4/liblmpi.a
ln -s LINUX/ch_p4/libmpe.a
ln -s LINUX/ch_p4/libmpe_nompi.a
ln -s LINUX/ch_p4/libmpich.a
ln -s LINUX/ch_p4/libpmpich.a
ln -s LINUX/ch_p4/libtmpi.a
popd

# install examples
mkdir -p $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/examples
cp -a $RPM_BUILD_DIR/mpich/examples \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}

pushd $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/examples
# remove this symlink
rm -f basic/upshot
# remove mpi2c++ examples as in separate package
rm -rf MPI-2-C++
# build correct makefiles
chmod 755 $RPM_BUILD_ROOT/usr/bin/mpireconfig
find . -name Makefile | xargs $RPM_BUILD_ROOT/usr/bin/mpireconfig
popd


# and the mpe examples
mkdir -p $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/examples/mpe
cp -a $RPM_BUILD_DIR/mpich/mpe/contrib/* \
      $RPM_BUILD_ROOT/usr/doc/mpich-%{PACKAGE_VERSION}/examples/mpe

# build gives an empty machine file: post install will replace this
cat /dev/null > $RPM_BUILD_ROOT/usr/util/machines/machines.LINUX

# add profile.d files
mkdir -p $RPM_BUILD_ROOT/etc/profile.d
cp %{SOURCE2} $RPM_BUILD_ROOT/etc/profile.d
cp %{SOURCE3} $RPM_BUILD_ROOT/etc/profile.d

%clean
rm -rf $RPM_BUILD_ROOT
# rm -rf $RPM_BUILD_DIR/mpich

%post
# if there user did not have a previous machines file, use this machine name
if [ ! -s /usr/util/machines/machines.LINUX ] ; then
    ( echo `hostname -f` ; echo `hostname -f` ; \
      echo `hostname -f` ; echo `hostname -f` ; \
      echo `hostname -f` ; echo `hostname -f` ; \
      echo `hostname -f` ; echo `hostname -f` ; \
    ) > /usr/util/machines/machines.LINUX
fi

%files
%attr(-, root, root) /usr/bin/execer
%attr(-, root, root) /usr/bin/mpiCC
%attr(-, root, root) /usr/bin/mpicc
%attr(-, root, root) /usr/bin/mpif77
%attr(-, root, root) /usr/bin/mpif90
%attr(-, root, root) /usr/bin/mpiman
%attr(-, root, root) /usr/bin/mpireconfig
%attr(-, root, root) /usr/bin/mpirun
%attr(-, root, root) /usr/bin/mpirun.args
%attr(-, root, root) /usr/bin/mpirun.ch_p4
%attr(-, root, root) /usr/bin/mpirun.pg
%attr(-, root, root) /usr/bin/serv_p4
%attr(-, root, root) /usr/bin/tarch
%attr(-, root, root) /usr/bin/tdevice
%attr(-, root, root) /usr/bin/tstmachines

%attr(-, root, root) /usr/include/basex11.h
%attr(-, root, root) /usr/include/binding.h
%attr(-, root, root) /usr/include/mpe.h
%attr(-, root, root) /usr/include/mpe_graphics.h
%attr(-, root, root) /usr/include/mpe_log.h
%attr(-, root, root) /usr/include/mpeexten.h
%attr(-, root, root) /usr/include/mpetools.h
%attr(-, root, root) /usr/include/mpi.h
%attr(-, root, root) /usr/include/mpi_errno.h
%attr(-, root, root) /usr/include/mpif.h
%attr(-, root, root) /usr/include/patchlevel.h
%attr(-, root, root) /usr/include/protofix.h

%attr(-, root, root) /usr/include/mpidefs.h
%attr(-, root, root) /usr/include/mpio.h

%attr(-, root, root) /usr/lib/LINUX/mpich.En_US.cat
%attr(-, root, root) /usr/lib/LINUX/mpich.cat

%attr(-, root, root) /usr/lib/libampi.a
%attr(-, root, root) /usr/lib/libfmpich.a
%attr(-, root, root) /usr/lib/liblmpi.a
%attr(-, root, root) /usr/lib/libmpe.a
%attr(-, root, root) /usr/lib/libmpe_nompi.a
%attr(-, root, root) /usr/lib/libmpich.a
%attr(-, root, root) /usr/lib/libpmpich.a
%attr(-, root, root) /usr/lib/libtmpi.a

%attr(755, root, root) /usr/lib/LINUX/ch_p4/chkserv
%attr(755, root, root) /usr/lib/LINUX/ch_p4/chp4_servs
%attr(755, root, root) /usr/lib/LINUX/ch_p4/config.status
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpiCC
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpicc
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpif77
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpif90
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpiman
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpireconfig
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpirun
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpirun.args
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpirun.ch_p4
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpirun.pg
%attr(755, root, root) /usr/lib/LINUX/ch_p4/serv_p4
%attr(755, root, root) /usr/lib/LINUX/ch_p4/tarch
%attr(755, root, root) /usr/lib/LINUX/ch_p4/tstmachines
%attr(755, root, root) /usr/lib/LINUX/ch_p4/mpif.h
%attr(-, root, root) /usr/lib/LINUX/ch_p4/mpichconf.h
%attr(-, root, root) /usr/lib/LINUX/ch_p4/mpidefs.h
%attr(-, root, root) /usr/lib/LINUX/ch_p4/mpio.h
%attr(-, root, root) /usr/lib/LINUX/ch_p4/mpiof.h
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libampi.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libfmpich.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/liblmpi.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libmpe.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libmpe_nompi.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libmpich.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libpmpich.a
%attr(-, root, root) /usr/lib/LINUX/ch_p4/libtmpi.a

%dir /usr/util/machines
%config %attr(-, root, root) /usr/util/machines/machines.LINUX

%attr(-, root, root) %doc /usr/man/man1/
%attr(-, root, root) %doc /usr/man/man3/
%attr(-, root, root) %doc /usr/man/man4/
%attr(-, root, root) %doc /usr/doc/mpich-%{PACKAGE_VERSION}/
